@@ -7,7 +7,7 @@ bess = function(x, y, family = c("gaussian", "binomial", "cox"),
                 glm.max = 1e6,
                 cox.max = 20,
                 epsilon = 1e-4,
-                normalize = FALSE)
+                normalize = TRUE)
 {
   family <- match.arg(family)
   if(ncol(x)==1|is.vector(x)) stop("x should be two columns at least!")
@@ -64,7 +64,7 @@ bess = function(x, y, family = c("gaussian", "binomial", "cox"),
       inactive = im[-ignores]
       normx[nosignal] = (.Machine$double.eps) * sqrt(n)
     }    else ignores = NULL
-    names(normx) = NULL
+    names(normx) = vn
     x = sqrt(n)*scale(x, FALSE, normx)
 
     #initial beta
@@ -73,23 +73,97 @@ bess = function(x, y, family = c("gaussian", "binomial", "cox"),
 
     if(method=="gsection")
     {
-     fit=abess_lm(x,y,s.min,s.max,K.max,max.steps,beta0,epsilon)
-     beta.fit=fit$beta[,1:fit$k]
-     mse=fit$mse[1:fit$k]
-     lambda=fit$lambda[1:fit$k]^2/2
-     nullmse=fit$nullmse
-     s.list=fit$s.list[1:fit$k]
+      k = 1
+      sL=s.min
+      sR=s.max
+      beta0R=beta0
+      beta0L=beta0
+      
+      fit_L1=bess.lm(x,y,
+                      beta0=beta0L,
+                      s=sL,
+                      max.steps=max.steps)
+      
+      nullmse=fit_L1$mse
+      fit_L=fit_L1
+      fit_R=bess.lm(x,y,
+                     beta0=beta0R,
+                     s=sR,
+                     max.steps=max.steps)
+      
+      beta.fit=cbind(fit_L$beta,fit_R$beta)
+      mse=c(fit_L$mse,fit_R$mse)
+      lambda=c(fit_L$lambda,fit_R$lambda)
+      aic=c(fit_L$AIC,fit_R$AIC)
+      bic=c(fit_L$BIC,fit_R$BIC)
+      
+      beta0M=fit_R$beta
+      s.list=c(sL,sR)
+      
+      while(k<=K.max)
+      {
+        sM <- round(sL + (sR-sL)*0.618)
+        s.list=c(s.list,sM)
+        fit_M=bess.lm(x,y,
+                       beta0=beta0M,
+                       s=sM,
+                       max.steps=max.steps)
+        cat(k,"-th iteration s.left:",sL," s.split:",sM," s.right:",sR,"\n",sep="")
+        
+        beta0M=fit_M$beta
+        beta.fit=cbind(beta.fit,beta0M)
+        
+        mse=c(mse, fit_M$mse)
+        lambda=c(lambda, fit_M$lambda)
+        aic=c(aic, fit_M$AIC)
+        bic=c(bic, fit_M$BIC)
+        if(abs(fit_L$mse-fit_M$mse)/abs(nullmse*(sM-sL)) > epsilon &
+           abs(fit_R$mse-fit_M$mse)/abs(nullmse*(sM-sR)) < epsilon)
+        {
+          sR <- sM
+          fit_R=fit_M
+        }else if(abs(fit_L$mse-fit_M$mse)/abs(nullmse) > epsilon &
+                 abs(fit_R$mse-fit_M$mse)/abs(nullmse) > epsilon)
+        {
+          sL <- sM
+          fit_L=fit_M
+        }else
+        {
+          sR=sM
+          fit_R=fit_M
+          sL=s.min
+          fit_L=fit_L1
+        }
+        
+        if(sR-sL==1) break
+        fit_ML=bess.lm(x,y,
+                        beta0=beta0M,
+                        s=sM,
+                        max.steps=max.steps)
+        
+        fit_MR=bess.lm(x,y,
+                        beta0=beta0M,
+                        s=sM,
+                        max.steps=max.steps)
+        if(abs(fit_ML$mse-fit_M$mse)/abs(fit_M$mse) > epsilon &
+           abs(fit_MR$mse-fit_M$mse)/abs(fit_M$mse) < epsilon)
+        {break}
+        
+        k=k+1
+      }
     }
     if(method=="sequential")
     {
       fit_L=bess.lm(x,y,s=s.list[length(s.list)],max.steps = max.steps,beta0=beta0)
       mse_L=fit_L$mse
       #cat(mse_L,"\n")
-      nullmse=sum(y^2)/(2*n)
+      nullmse=sum(y^2)/n
       #cat(nullmse,"\n")
       beta.fit=matrix(beta0,m,1)
       mse=vector()
       lambda=vector()
+      aic=vector()
+      bic=vector()
       for(k in 1:length(s.list))
       {
         cat("select",s.list[k],"variables","\n")
@@ -100,7 +174,9 @@ bess = function(x, y, family = c("gaussian", "binomial", "cox"),
         mse[k]=fit$mse
         #lambda
         lambda[k]=fit$lambda
-
+        aic[k]=fit$AIC
+        bic[k]=fit$BIC
+        
         #cat(abs((mse[k]-mse_L)/((nullmse-mse_L)*(L-s.list[k]))),"\n")
         if(s.list[k]==s.list[length(s.list)]) break
         if(abs((mse[k]-mse_L)/((nullmse-mse_L)*(s.list[length(s.list)]-s.list[k])))<epsilon) break
@@ -111,11 +187,14 @@ bess = function(x, y, family = c("gaussian", "binomial", "cox"),
 
 
     beta.fit=sqrt(n)*(beta.fit)/normx
-    coef0=mu-sum(beta.fit*meanx)
-
+    colnames(beta.fit) = s.list
+    rownames(beta.fit) = vn
+    coef0=mu-drop(t(beta.fit)%*%meanx)
+    names(coef0)=s.list
+    
     out=list(family="bess_gaussian",beta=beta.fit,coef0=coef0,
              s.list=s.list,meanx=meanx,normx=normx,meany=mu,
-             mse=mse,nullmse=nullmse,lambda=lambda)
+             mse=mse,nullmse=nullmse,AIC=aic,BIC=bic,lambda=lambda)
     class(out)="bess"
     return(out)
   }
@@ -129,7 +208,7 @@ bess = function(x, y, family = c("gaussian", "binomial", "cox"),
       inactive = im[-ignores]
       normx[nosignal] = (.Machine$double.eps) * sqrt(n)
     }    else ignores = NULL
-    names(normx) = NULL
+    names(normx) = vn
     x = sqrt(n)*scale(x, FALSE, normx)
 
     beta0=rep(0,m)
@@ -149,7 +228,7 @@ bess = function(x, y, family = c("gaussian", "binomial", "cox"),
     fit_L1=bess.glm(x=x,y=y,
                     beta0=beta0L,
                     intercept=coef0L,
-                    s=s.list[length(s.list)],
+                    s=sL,
                     glm.max=glm.max,
                     max.steps=max.steps)
 
@@ -171,6 +250,8 @@ bess = function(x, y, family = c("gaussian", "binomial", "cox"),
     beta0M=fit_R$beta
     coef0M=fit_R$coef0
     s.list=c(sL,sR)
+    aic=c(fit_L$AIC,fit_R$AIC)
+    bic=c(fit_L$BIC,fit_R$BIC)
     while(k<=K.max)
     {
       sM <- round(sL + (sR-sL)*0.618)
@@ -191,6 +272,9 @@ bess = function(x, y, family = c("gaussian", "binomial", "cox"),
 
       dev=c(dev,fit_M$deviance)
       lambda=c(lambda,fit_M$lambda)
+      aic=c(aic, fit_M$AIC)
+      bic=c(bic, fit_M$BIC)
+      
       if(abs(fit_L$deviance-fit_M$deviance)/abs(nulldev*(sM-sL)) > epsilon &
          abs(fit_R$deviance-fit_M$deviance)/abs(nulldev*(sM-sR)) < epsilon)
       {
@@ -246,6 +330,8 @@ bess = function(x, y, family = c("gaussian", "binomial", "cox"),
       if(abs(dev_L/nulldev)>0.5) dev_L=0
       dev=vector()
       lambda=vector()
+      aic=vector()
+      bic=vector()
       for(k in 1:length(s.list))
       {
         cat("select",s.list[k],"variables","\n")
@@ -255,8 +341,9 @@ bess = function(x, y, family = c("gaussian", "binomial", "cox"),
                           max.steps=max.steps)
 
         dev[k]=fit$deviance
-
         lambda[k]=fit$lambda
+        aic[k]=fit$AIC
+        bic[k]=fit$BIC
         beta.fit=cbind(beta.fit,fit$beta)
         coef0.fit=c(coef0.fit,fit$coef0)
         #cat(abs((dev[k]-dev_L)/((s.list[length(s.list)]-s.list[k])*(nulldev))),"\n")
@@ -270,19 +357,21 @@ bess = function(x, y, family = c("gaussian", "binomial", "cox"),
     }
 
     beta.fit=sqrt(n)*(beta.fit)/normx
-    coef0.fit=coef0.fit-drop(t(beta.fit)%*%meanx)
-
+    colnames(beta.fit) = s.list 
+    rownames(beta.fit) = vn
+    coef0.fit = coef0.fit-drop(t(beta.fit)%*%meanx)
+    names(coef0.fit) = s.list
 
     if(!setequal(y_names,c(0,1)))
     {
       out=list(family="bess_binomial",beta=beta.fit,coef0=coef0.fit,s.list=s.list,
-         meanx=meanx,normx=normx,dev=dev,nulldev=nulldev,lambda=lambda,y_names=y_names)
+         meanx=meanx,normx=normx,deviance=dev,nulldev=nulldev,AIC=aic,BIC=bic,lambda=lambda,y_names=y_names)
       class(out)="bess"
       return(out)
     }else
     {
       out=list(family="bess_binomial",beta=beta.fit,coef0=coef0.fit,s.list=s.list,
-               meanx=meanx,normx=normx,deviance=dev,nulldeviance=nulldev,lambda=lambda)
+               meanx=meanx,normx=normx,deviance=dev,nulldeviance=nulldev,AIC=aic,BIC=bic,lambda=lambda)
       class(out)="bess"
       return(out)
     }
@@ -301,7 +390,7 @@ bess = function(x, y, family = c("gaussian", "binomial", "cox"),
       inactive = im[-ignores]
       normx[nosignal] = (.Machine$double.eps) * sqrt(n)
     }    else ignores = NULL
-    names(normx) = NULL
+    names(normx) = vn
     x = sqrt(n)*scale(x, FALSE, normx)
     x=x[mark,]
 
@@ -336,7 +425,8 @@ bess = function(x, y, family = c("gaussian", "binomial", "cox"),
 
     beta0M=fit_R$beta
     s.list=c(sL,sR)
-
+    aic=c(fit_L$AIC,fit_R$AIC)
+    bic=c(fit_L$BIC,fit_R$BIC)
     while(k<=K.max)
     {
       sM <- round(sL + (sR-sL)*0.618)
@@ -353,6 +443,8 @@ bess = function(x, y, family = c("gaussian", "binomial", "cox"),
 
       dev=c(dev,fit_M$deviance)
       lambda=c(lambda,fit_M$lambda)
+      aic=c(aic, fit_M$AIC)
+      bic=c(bic, fit_M$BIC)
       if(abs(fit_L$deviance-fit_M$deviance)/abs(nulldev*(sM-sL)) > epsilon &
          abs(fit_R$deviance-fit_M$deviance)/abs(nulldev*(sM-sR)) < epsilon)
       {
@@ -402,7 +494,8 @@ bess = function(x, y, family = c("gaussian", "binomial", "cox"),
 
       dev=vector()
       lambda=vector()
-
+      aic=vector()
+      bic=vector()
       for(k in 1:length(s.list))
       {
         cat("select",s.list[k],"variables","\n")
@@ -415,7 +508,8 @@ bess = function(x, y, family = c("gaussian", "binomial", "cox"),
         #cat(dev[k],"\n")
         lambda[k]=fit$lambda
         beta.fit=cbind(beta.fit,fit$beta)
-
+        aic[k]=fit$AIC
+        bic[k]=fit$BIC
 
         #cat(abs((dev[k]-dev_L)/((nulldev-dev_L)*(s.list[length(s.list)]-s.list[k]))),"\n")
         if(s.list[k]==s.list[length(s.list)]) break
@@ -427,9 +521,11 @@ bess = function(x, y, family = c("gaussian", "binomial", "cox"),
     }
 
     beta.fit=sqrt(n)*(beta.fit)/normx
-
+    colnames(beta.fit) = s.list
+    rownames(beta.fit) = vn
+    
     out=list(family="bess_cox",beta=beta.fit,s.list=s.list,meanx=meanx,
-             normx=normx,deviance=dev,nulldeviance=nulldev,lambda=lambda)
+             normx=normx,deviance=dev,nulldeviance=nulldev,AIC=aic,BIC=bic,lambda=lambda)
     class(out)="bess"
     return(out)
   }
